@@ -17,15 +17,12 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #include <inttypes.h>
 
-typedef struct pa_source_output pa_source_output;
-
+#include <pulsecore/typedefs.h>
 #include <pulse/sample.h>
 #include <pulse/format.h>
 #include <pulsecore/memblockq.h>
@@ -43,7 +40,7 @@ typedef enum pa_source_output_state {
     PA_SOURCE_OUTPUT_UNLINKED
 } pa_source_output_state_t;
 
-static inline pa_bool_t PA_SOURCE_OUTPUT_IS_LINKED(pa_source_output_state_t x) {
+static inline bool PA_SOURCE_OUTPUT_IS_LINKED(pa_source_output_state_t x) {
     return x == PA_SOURCE_OUTPUT_RUNNING || x == PA_SOURCE_OUTPUT_CORKED;
 }
 
@@ -78,6 +75,15 @@ struct pa_source_output {
     pa_client *client;                    /* may be NULL */
 
     pa_source *source;                    /* NULL while being moved */
+
+    /* This is set to true when creating the source output if the source was
+     * requested by the application that created the source output. This is
+     * sometimes useful for determining whether the source output should be
+     * moved by some automatic policy. If the source output is moved away from
+     * the source that the application requested, this flag is reset to
+     * false. */
+    bool source_requested_by_application;
+
     pa_source *destination_source;        /* only set by filter sources */
 
     /* A source output can monitor just a single input of a sink, in which case we find it here */
@@ -87,7 +93,7 @@ struct pa_source_output {
     pa_channel_map channel_map;
     pa_format_info *format;
 
-    /* Also see http://pulseaudio.org/wiki/InternalVolumes */
+    /* Also see http://www.freedesktop.org/wiki/Software/PulseAudio/Documentation/Developer/Volumes/ */
     pa_cvolume volume;             /* The volume clients are informed about */
     pa_cvolume reference_ratio;    /* The ratio of the stream's volume to the source's reference volume */
     pa_cvolume real_ratio;         /* The ratio of the stream's volume to the source's real volume */
@@ -96,15 +102,19 @@ struct pa_source_output {
 
     pa_cvolume volume_factor_source; /* A second volume factor in format of the source this stream is connected to */
 
-    pa_bool_t volume_writable:1;
+    bool volume_writable:1;
 
-    pa_bool_t muted:1;
+    bool muted:1;
 
-    /* if TRUE then the source we are connected to and/or the volume
-     * set is worth remembering, i.e. was explicitly chosen by the
-     * user and not automatically. module-stream-restore looks for
-     * this.*/
-    pa_bool_t save_source:1, save_volume:1, save_muted:1;
+    /* if true then the volume and the mute state of this source-output
+     * are worth remembering, module-stream-restore looks for this. */
+    bool save_volume:1, save_muted:1;
+
+    /* if users move the source-output to a source, and the source is not
+     * default_source, the source->name will be saved in preferred_source. And
+     * later if source-output is moved to other sources for some reason, it
+     * still can be restored to the preferred_source at an appropriate time */
+    char *preferred_source;
 
     pa_resample_method_t requested_resample_method, actual_resample_method;
 
@@ -144,12 +154,13 @@ struct pa_source_output {
     void (*detach) (pa_source_output *o);           /* may be NULL */
 
     /* If non-NULL called whenever the source this output is attached
-     * to suspends or resumes. Called from main context */
-    void (*suspend) (pa_source_output *o, pa_bool_t b);   /* may be NULL */
+     * to suspends or resumes or if the suspend cause changes.
+     * Called from main context */
+    void (*suspend) (pa_source_output *o, pa_source_state_t old_state, pa_suspend_cause_t old_suspend_cause);   /* may be NULL */
 
     /* If non-NULL called whenever the source this output is attached
      * to suspends or resumes. Called from IO context */
-    void (*suspend_within_thread) (pa_source_output *o, pa_bool_t b);   /* may be NULL */
+    void (*suspend_within_thread) (pa_source_output *o, bool b);   /* may be NULL */
 
     /* If non-NULL called whenever the source output is moved to a new
      * source. Called from main context after the source output has been
@@ -174,9 +185,9 @@ struct pa_source_output {
     void (*state_change) (pa_source_output *o, pa_source_output_state_t state); /* may be NULL */
 
     /* If non-NULL this function is called before this source output
-     * is moved to a source and if it returns FALSE the move
+     * is moved to a source and if it returns false the move
      * will not be allowed */
-    pa_bool_t (*may_move_to) (pa_source_output *o, pa_source *s); /* may be NULL */
+    bool (*may_move_to) (pa_source_output *o, pa_source *s); /* may be NULL */
 
     /* If non-NULL this function is used to dispatch asynchronous
      * control events. */
@@ -194,9 +205,9 @@ struct pa_source_output {
         pa_source_output_state_t state;
 
         pa_cvolume soft_volume;
-        pa_bool_t muted:1;
+        bool muted:1;
 
-        pa_bool_t attached:1; /* True only between ->attach() and ->detach() calls */
+        bool attached:1; /* True only between ->attach() and ->detach() calls */
 
         pa_sample_spec sample_spec;
 
@@ -246,6 +257,7 @@ typedef struct pa_source_output_new_data {
     pa_client *client;
 
     pa_source *source;
+    bool source_requested_by_application;
     pa_source *destination_source;
 
     pa_resample_method_t resample_method;
@@ -257,31 +269,33 @@ typedef struct pa_source_output_new_data {
     pa_idxset *nego_formats;
 
     pa_cvolume volume, volume_factor, volume_factor_source;
-    pa_bool_t muted:1;
+    bool muted:1;
 
-    pa_bool_t sample_spec_is_set:1;
-    pa_bool_t channel_map_is_set:1;
+    bool sample_spec_is_set:1;
+    bool channel_map_is_set:1;
 
-    pa_bool_t volume_is_set:1, volume_factor_is_set:1, volume_factor_source_is_set:1;
-    pa_bool_t muted_is_set:1;
+    bool volume_is_set:1, volume_factor_is_set:1, volume_factor_source_is_set:1;
+    bool muted_is_set:1;
 
-    pa_bool_t volume_is_absolute:1;
+    bool volume_is_absolute:1;
 
-    pa_bool_t volume_writable:1;
+    bool volume_writable:1;
 
-    pa_bool_t save_source:1, save_volume:1, save_muted:1;
+    bool save_volume:1, save_muted:1;
+    char *preferred_source;
 } pa_source_output_new_data;
 
 pa_source_output_new_data* pa_source_output_new_data_init(pa_source_output_new_data *data);
 void pa_source_output_new_data_set_sample_spec(pa_source_output_new_data *data, const pa_sample_spec *spec);
 void pa_source_output_new_data_set_channel_map(pa_source_output_new_data *data, const pa_channel_map *map);
-pa_bool_t pa_source_output_new_data_is_passthrough(pa_source_output_new_data *data);
+bool pa_source_output_new_data_is_passthrough(pa_source_output_new_data *data);
 void pa_source_output_new_data_set_volume(pa_source_output_new_data *data, const pa_cvolume *volume);
 void pa_source_output_new_data_apply_volume_factor(pa_source_output_new_data *data, const pa_cvolume *volume_factor);
 void pa_source_output_new_data_apply_volume_factor_source(pa_source_output_new_data *data, const pa_cvolume *volume_factor);
-void pa_source_output_new_data_set_muted(pa_source_output_new_data *data, pa_bool_t mute);
-pa_bool_t pa_source_output_new_data_set_source(pa_source_output_new_data *data, pa_source *s, pa_bool_t save);
-pa_bool_t pa_source_output_new_data_set_formats(pa_source_output_new_data *data, pa_idxset *formats);
+void pa_source_output_new_data_set_muted(pa_source_output_new_data *data, bool mute);
+bool pa_source_output_new_data_set_source(pa_source_output_new_data *data, pa_source *s, bool save,
+                                          bool requested_by_application);
+bool pa_source_output_new_data_set_formats(pa_source_output_new_data *data, pa_idxset *formats);
 void pa_source_output_new_data_done(pa_source_output_new_data *data);
 
 /* To be called by the implementing module only */
@@ -294,13 +308,12 @@ int pa_source_output_new(
 void pa_source_output_put(pa_source_output *o);
 void pa_source_output_unlink(pa_source_output*o);
 
-void pa_source_output_set_name(pa_source_output *o, const char *name);
-
 pa_usec_t pa_source_output_set_requested_latency(pa_source_output *o, pa_usec_t usec);
 
-void pa_source_output_cork(pa_source_output *o, pa_bool_t b);
+void pa_source_output_cork(pa_source_output *o, bool b);
 
 int pa_source_output_set_rate(pa_source_output *o, uint32_t rate);
+int pa_source_output_update_resampler(pa_source_output *o);
 
 size_t pa_source_output_get_max_rewind(pa_source_output *o);
 
@@ -311,33 +324,32 @@ void pa_source_output_kill(pa_source_output*o);
 
 pa_usec_t pa_source_output_get_latency(pa_source_output *o, pa_usec_t *source_latency);
 
-pa_bool_t pa_source_output_is_volume_readable(pa_source_output *o);
-pa_bool_t pa_source_output_is_passthrough(pa_source_output *o);
-void pa_source_output_set_volume(pa_source_output *o, const pa_cvolume *volume, pa_bool_t save, pa_bool_t absolute);
-pa_cvolume *pa_source_output_get_volume(pa_source_output *o, pa_cvolume *volume, pa_bool_t absolute);
+bool pa_source_output_is_volume_readable(pa_source_output *o);
+bool pa_source_output_is_passthrough(pa_source_output *o);
+void pa_source_output_set_volume(pa_source_output *o, const pa_cvolume *volume, bool save, bool absolute);
+pa_cvolume *pa_source_output_get_volume(pa_source_output *o, pa_cvolume *volume, bool absolute);
 
-void pa_source_output_set_mute(pa_source_output *o, pa_bool_t mute, pa_bool_t save);
-pa_bool_t pa_source_output_get_mute(pa_source_output *o);
+void pa_source_output_set_mute(pa_source_output *o, bool mute, bool save);
 
+void pa_source_output_set_property(pa_source_output *o, const char *key, const char *value);
+void pa_source_output_set_property_arbitrary(pa_source_output *o, const char *key, const uint8_t *value, size_t nbytes);
 void pa_source_output_update_proplist(pa_source_output *o, pa_update_mode_t mode, pa_proplist *p);
 
 pa_resample_method_t pa_source_output_get_resample_method(pa_source_output *o);
 
 void pa_source_output_send_event(pa_source_output *o, const char *name, pa_proplist *data);
 
-pa_bool_t pa_source_output_may_move(pa_source_output *o);
-pa_bool_t pa_source_output_may_move_to(pa_source_output *o, pa_source *dest);
-int pa_source_output_move_to(pa_source_output *o, pa_source *dest, pa_bool_t save);
+bool pa_source_output_may_move(pa_source_output *o);
+bool pa_source_output_may_move_to(pa_source_output *o, pa_source *dest);
+int pa_source_output_move_to(pa_source_output *o, pa_source *dest, bool save);
 
 /* The same as pa_source_output_move_to() but in two separate steps,
  * first the detaching from the old source, then the attaching to the
  * new source */
 int pa_source_output_start_move(pa_source_output *o);
-int pa_source_output_finish_move(pa_source_output *o, pa_source *dest, pa_bool_t save);
+int pa_source_output_finish_move(pa_source_output *o, pa_source *dest, bool save);
 void pa_source_output_fail_move(pa_source_output *o);
 
-#define pa_source_output_get_state(o) ((o)->state)
-
 pa_usec_t pa_source_output_get_requested_latency(pa_source_output *o);
 
 /* To be used exclusively by the source driver thread */
@@ -352,6 +364,46 @@ int pa_source_output_process_msg(pa_msgobject *mo, int code, void *userdata, int
 
 pa_usec_t pa_source_output_set_requested_latency_within_thread(pa_source_output *o, pa_usec_t usec);
 
+/* Calls the attach() callback if it's set. The output must be in detached
+ * state. */
+void pa_source_output_attach(pa_source_output *o);
+
+/* Calls the detach() callback if it's set and the output is attached. The
+ * output is allowed to be already detached, in which case this does nothing.
+ *
+ * The reason why this can be called for already-detached outputs is that when
+ * a filter source's output is detached, it has to detach also all outputs
+ * connected to the filter source. In case the filter source's output was
+ * detached because the filter source is being removed, those other outputs
+ * will be moved to another source or removed, and moving and removing involve
+ * detaching the outputs, but the outputs at that point are already detached.
+ *
+ * XXX: Moving or removing an output also involves sending messages to the
+ * output's source. If the output's source is a detached filter source,
+ * shouldn't sending messages to it be prohibited? The messages are processed
+ * in the root source's IO thread, and when the filter source is detached, it
+ * would seem logical to prohibit any interaction with the IO thread that isn't
+ * any more associated with the filter source. Currently sending messages to
+ * detached filter sources mostly works, because the filter sources don't
+ * update their asyncmsgq pointer when detaching, so messages still find their
+ * way to the old IO thread. */
+void pa_source_output_detach(pa_source_output *o);
+
+/* Called from the main thread, from source.c only. The normal way to set the
+ * source output volume is to call pa_source_output_set_volume(), but the flat
+ * volume logic in source.c needs also a function that doesn't do all the extra
+ * stuff that pa_source_output_set_volume() does. This function simply sets
+ * o->volume and fires change notifications. */
+void pa_source_output_set_volume_direct(pa_source_output *o, const pa_cvolume *volume);
+
+/* Called from the main thread, from source.c only. This shouldn't be a public
+ * function, but the flat volume logic in source.c currently needs a way to
+ * directly set the source output reference ratio. This function simply sets
+ * o->reference_ratio and logs a message if the value changes. */
+void pa_source_output_set_reference_ratio(pa_source_output *o, const pa_cvolume *ratio);
+
+void pa_source_output_set_preferred_source(pa_source_output *o, pa_source *s);
+
 #define pa_source_output_assert_io_context(s) \
     pa_assert(pa_thread_mq_get() || !PA_SOURCE_OUTPUT_IS_LINKED((s)->state))
 
